/*
	MERVBot Spawn-callback by Catid@pacbell.net
*/

#ifndef SPAWN_H
#define SPAWN_H


#include "..\dllcore.h"
#include <windows.h>


union itemInfo
{
	struct
	{
		unsigned shields	: 1;	// Has it or not
		unsigned supers		: 1;
		unsigned burst		: 4;	// Item counts
		unsigned repel		: 4;
		unsigned thor		: 4;
		unsigned brick		: 4;
		unsigned decoy		: 4;
		unsigned rocket		: 4;
		unsigned portal		: 4;
		unsigned pack		: 2;	// Probably used somehow
	};

	Uint32 n;
};

union weaponInfo
{
	struct
	{
		unsigned type		: 5;	// enum Projectile_Types
		unsigned level		: 2;	// Only for bombs/bullets
		unsigned bouncing	: 1;	// Only for bombs
		unsigned emp		: 1;	// Only for bombs
		unsigned isBomb		: 1;	// ? This is a bomb
		unsigned shrapCount	: 5;	// 0-31
		unsigned fireType	: 1;	// Bombs -> Mines, Bullets -> Multifire
	};

	Uint16 n;
};

enum Projectile_Types
{
	// Seen "in the wild"
	PROJ_None,
	PROJ_Bullet,
	PROJ_BBullet,
	PROJ_Bomb,
	PROJ_PBomb,
	PROJ_Repel,
	PROJ_Decoy,
	PROJ_Burst,
	PROJ_Thor,

	// Internal to the bot
	PROJ_InactiveBullet,
	PROJ_Shrapnel
};

enum Weapon_Levels
{
	LVL_One,
	LVL_Two,
	LVL_Three,
	LVL_Four
};

enum Ship_Types
{
	SHIP_Warbird,		// 0
	SHIP_Javelin,
	SHIP_Spider,
	SHIP_Leviathan,
	SHIP_Terrier,
	SHIP_Weasel,
	SHIP_Lancaster,
	SHIP_Shark,
	SHIP_Spectator		// 8
};

enum Prize_Types
{
	PRIZE_Unknown,		// 0
	PRIZE_Recharge,
	PRIZE_Energy,
	PRIZE_Rotation,
	PRIZE_Stealth,
	PRIZE_Cloak,
	PRIZE_XRadar,
	PRIZE_Warp,
	PRIZE_Guns,
	PRIZE_Bombs,
	PRIZE_BBullets,
	PRIZE_Thruster,
	PRIZE_TopSpeed,
	PRIZE_FullCharge,
	PRIZE_EngineShutdown,
	PRIZE_Multifire,
	PRIZE_Proximity,
	PRIZE_Super,
	PRIZE_Shields,
	PRIZE_Antiwarp,
	PRIZE_Repel,
	PRIZE_Burst,
	PRIZE_Decoy,
	PRIZE_Thor,
	PRIZE_Multiprize,
	PRIZE_Brick,
	PRIZE_Rocket,
	PRIZE_Portal,		// 28
};

enum Chat_Modes
{
    MSG_Arena,			// 00 Same for *arena, *zone, *szone, **
    MSG_PublicMacro,	// 01 Macro calcs are done client-side
    MSG_Public,			// 02 Public chat
    MSG_Team,			// 03 (//) or (')
    MSG_TeamPrivate,	// 04 Player to all members of another team (")
    MSG_Private,		// 05 Only shows messages addressed to the bot
    MSG_PlayerWarning,	// 06 Red message, with a name tag
    MSG_RemotePrivate,	// 07 Do not trust the sender with any private information.
    MSG_ServerError,	// 08 Red server errors, without a name tag
    MSG_Channel			// 09 Arrives in the same format SubSpace displays
};

#define VEX_TEAMFILE "MERVBot.teams"
#define VEX_XORKEY 0xF6383684
#define VEX_INVITE_EXPIRE_DELAY 5

#include <sys/stat.h>
#include <time.h>

#pragma pack(4)

struct TeamEntry
{
	Uint16	Freq;
	char	Owner[20];
	char	GreetMessage[200];
	Uint32	Password;
	char	AllowedSquad[20];
	char	Name[32];
	time_t	LastUsed;

	TeamEntry()
	{
		Freq = 0;
		Password = 0;
		LastUsed = 0;
		ZeroMemory(Owner, 20);
		ZeroMemory(GreetMessage, 200);
		ZeroMemory(AllowedSquad, 20);
		ZeroMemory(Name, 32);
	}
};

#pragma pack()

struct TeamInviteEntry
{
	char Name[20];
	Uint32 Timestamp;
	Uint16 Freq;
};

class TeamBox
{
	inline void DoXOR(char *buffer, Uint32 len);

public:
	_linkedlist <TeamEntry> Box;
	_linkedlist <TeamInviteEntry> InviteBox;

	/// Main functions ///
	TeamEntry *AddTeam(Uint16 Freq, char *Owner, Uint32 Password);

	inline void RemoveTeam(TeamEntry *t)
	{
		Box.kill(t);
	}

	Uint32 CountOwned(char *Owner);

	/// Invite functions ///
	TeamInviteEntry *AddInvite(Uint16 Freq, char *Name);

	inline void RemoveInvite(TeamInviteEntry *t)
	{
		InviteBox.kill(t);
	}

	void ExpireInvites(Uint32 Seconds);

	/// Find functions ///
	TeamEntry *FindTeam(Uint16 Freq);
	TeamEntry *FindTeam(char *Owner);
	TeamInviteEntry *FindInvite(Uint16 Freq, char *Name);

	/// File IO ///
	bool LoadFromDisk();
	bool SaveToDisk();

	/// Constructors/Destructors ///
	TeamBox()
	{
		LoadFromDisk();
	}

	~TeamBox()
	{
		SaveToDisk();
	}
};

class botInfo
{
	bool CONNECTION_DENIED;

	CALL_HANDLE handle;
	CALL_COMMAND callback;
	CALL_PLIST playerlist;
	CALL_FLIST flaglist;
	char *arena;
	arenaSettings *settings;
	Player *me;
	BYTE *map;

	int countdown[4];

	/************************/

	Uint16 SpecFreq;

	Uint32 MaxOwnedFreqs;
	Uint32 InviteExpireSeconds;
	Uint32 AutoSaveDelay;

	/************************/

public:
	botInfo(CALL_HANDLE given)
	{
		handle = given;
		callback = 0;
		playerlist = 0;
		countdown[0] = VEX_INVITE_EXPIRE_DELAY;	// Invite expiring
		countdown[1] = 0;	// Autosave
		countdown[2] = 0;
		countdown[3] = 0;
		CONNECTION_DENIED = false;
		arena = "<Unknown>";
		SpecFreq = 0;
		MaxOwnedFreqs = 0;
		InviteExpireSeconds = 0;
		AutoSaveDelay = 0;
	}

	/************************/

	void LoadSettings();

	Uint32 strFirstInstanceOf(char *msg, char *msg2);		// Where msg2 begins in msg
	String strReplace(char *msg, char *from, char *to);		// Replace all instances of from in msg with to

	String getDateString(tm *CDate);	// [String] - Get the date from a tm
	String getTimeString(tm *CDate);	// [String] - Get the time from a tm

	/************************/

	void gotEvent(BotEvent &event);

	void tell(BotEvent &event);

	bool validate(CALL_HANDLE given) { return given == handle; }

	void sendPrivate(Player *player, char *msg);
	void sendPrivate(Player *player, BYTE snd, char *msg);

	void sendTeam(char *msg);
	void sendTeam(BYTE snd, char *msg);

	void sendTeamPrivate(Uint16 team, char *msg);
	void sendTeamPrivate(Uint16 team, BYTE snd, char *msg);

	void sendPublic(char *msg);
	void sendPublic(BYTE snd, char *msg);

	void sendPublicMacro(char *msg);
	void sendPublicMacro(BYTE snd, char *msg);

	void sendChannel(char *msg);			// #;Message
	void sendRemotePrivate(char *msg);		// :Name:Messsage
	void sendRemotePrivate(char *name, char *msg);

	void gotHelp(Player *p, Command *c);
	void gotCommand(Player *p, Command *c);
	void gotRemoteHelp(char *p, Command *c, Operator_Level l);
	void gotRemote(char *p, Command *c, Operator_Level l);
};


// The botlist contains every bot to ever be spawned of this type,
// some entries may no longer exist.

extern _linkedlist <botInfo> botlist;

botInfo *findBot(CALL_HANDLE handle);


#endif	// SPAWN_H
